RenderMultidrawableBatchSet

Struct RenderMultidrawableBatchSet 

pub struct RenderMultidrawableBatchSet<BPI>
where BPI: BinnedPhaseItem,
{ /* private fields */ }
A collection of mesh instances that can be drawn together, sorted into bins.

This data structure stores a list of entity indices corresponding to mesh instances, along with the bins they live in. Each bin contains the offset of the indirect parameters needed to draw that bin.

This data structure consists of both CPU and GPU parts. A schematic diagram of the data structure is as follows:

        ┌─
        │                ─────┬──────────────┬─────
        │                     │ Mesh Inst. 2 │
        │  Binned Mesh    ... ├──────────────┤ ...
        │  Instances          │ Entity 8     │
        │                ─────┴───┬──────────┴─────
        │                         │
        │                         │   ┌───────────────────────────────┐
        │                         │   │                               │
        │                         ▼   ▼                               │
        │               ┌───────┬───────┬───────┬─────                │
        │  Bins         │ Bin 0 │ Bin 1 │ Bin 2 │ ...                 │
        │               └───────┴───┬───┴───────┴─────                │
        │                           │                                 │
    CPU │                           │                                 │
        │  Entity-to-               │  ┌──────────┬──────────┬─────   │
        │  Binned-Mesh-             └─►│ Entity 3 │ Entity 8 │ ...    │
        │  Instance-                   └──────────┴──────┬───┴─────   │
        │  Index                                         │            │
        │                                                │            │
        │                                                │            │
        │                                                │            │
        │                                                │            │
        │  Indirect-     ┌───────┬───────┬───────┬─────  │            │
        │  Parameters-   │ IPO 0 │ IPO 1 │ IPO 2 │ ...   │            │
        │  Offset-to-    └───────┴───────┴───────┴─────  │            │
        │  Bin-Index                         ▲           │            │
        │                                    │           │            │
        └─                                   │           │            │
                                     ┌───────┘           │            │
        ┌─                           │                   │            │
        │                            │                   │            │
        │  Bin-to-                   ▼                   │            │
        │  Indirect-     ┌───────┬───────┬───────┬─────  │            │
        │  Parameters-   │ Bin 0 │ Bin 1 │ Bin 2 │ ...   │            │
        │  Offset        └───────┴───────┴───────┴─────  │            │
    GPU │  Buffer                                        │            │
        │                                                │            │
        │                                                ▼            │
        │  Binned Mesh   ─────┬──────────────┬──────────────┬─────    │
        │  Instance       ... │ Mesh Inst. 1 │ Mesh Inst. 2 │ ...     │
        │  Buffer        ─────┴──────────────┴───────────┬──┴─────    │
        │                                                │            │
        └─                                               └────────────┘